import numpy as np
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt

def getDetector(prefix):
    return prefix.split('/')[1].split('-')[0]

def plotHellinger(prefix):
    D_hellinger = np.genfromtxt(prefix+"imageHist.csv")
    plt.hist(D_hellinger, bins=100, normed=0, stacked=0, color='0.75')
    plt.xlabel('Distance')
    plt.ylabel('Count')
    plt.title('Hellinger distances')
    plt.axis([0,0.4,0,700])

def plotNorms(prefix):
    D_hellinger = np.genfromtxt(prefix+"imageDiff.csv")
    plt.hist(D_hellinger, bins=100, normed=0, stacked=0, color='0.75')
    plt.xlabel('Norm')
    plt.ylabel('Count')
    plt.title('Norm of difference')
    plt.axis([0,20000,0,700])

def validErrorIdx(errors, maxErr):
    return np.logical_and(errors > 0, errors < maxErr)

def plotReprojectionErrors(prefix):
    errors = np.genfromtxt(prefix+"-reprojectionErrors.csv")
    errors = errors[validErrorIdx(errors,10)]
    plt.hist(errors, bins=100, normed=0, stacked=0, color='0.75')
    plt.xlabel('Errors')
    plt.ylabel('Count')
    plt.title('Reprojection Errors')
    plt.show()

def getTimeStats(prefix):
    timing = np.genfromtxt(prefix+'timing.csv');
    return timing

def getKeypointStats(prefix):
    numKeypoints = np.genfromtxt(prefix+"-nKeypoints.csv")
    maxKeypoints = np.genfromtxt(prefix+"-mKeypoints.csv")
    return (numKeypoints, maxKeypoints)

def getMatchingStats(prefix):
    errors = np.genfromtxt(prefix+"-reprojectionErrors.csv")
    errors = np.ma.fix_invalid(errors).data
    matches = np.genfromtxt(prefix+"-nMatches.csv")
    return (errors, matches)

def printStats(prefix):
    timing = getTimeStats(prefix)
    (numKeypoints, maxKeypoints) = getKeypointStats(prefix)
    (errors, matches) = getMatchingStats(prefix)
    erridx = validErrorIdx(errors, 5)
    print(prefix + " matches: %d, homo: %d, mean time: %.2e" 
            % (matches.size,np.sum(erridx),np.mean(timing)))

def plotKeypointStats(prefix):
    (numKeypoints, maxKeypoints) = getKeypointStats(prefix)
    sz = maxKeypoints.size
    imageRange = np.linspace(1, sz, sz)
    plt.plot(imageRange, numKeypoints/10, imageRange, maxKeypoints)
    plt.show()

def plotHomoStats(prefix, startI=0, maxData=1000):
    (errors, matches) = getMatchingStats(prefix)
    errors = errors[startI:startI+maxData]
    errIdx = validErrorIdx(errors, 5)
    matches = matches[startI:startI+maxData]
    sz = maxData
    matchRange = np.linspace(1,sz,sz)
    print('errsz: %f, sum errs: %f, match sz: %f' % (errors.size, np.sum(errIdx), sz))

    plotVerticalSpans(errIdx, idxoffset=1, scale=1.1)
    plt.plot(matchRange, matches)
    plt.xlabel('Frame pair index')
    plt.ylabel('Feature matches')
    plt.title('Matches and correspondences ' + getDetector(prefix))
    plt.xlim((1,maxData))
    plt.ylim((0,100))

def plotVerticalSpans(mask, idxoffset=0, scale=1, facecolor='0.5'):
    for idx, val in enumerate(mask):
        if val == True:
            plt.axvspan(scale*idx+idxoffset, scale*(idx+1)+idxoffset, facecolor=facecolor, edgecolor='none')


def testSpan():
    l = np.linspace(0,9,10)
    b = np.array([False, False, False, True, True, False, True, False, False, False])
    y = np.sin(l)

    plotVerticalSpans(b) 

    plt.plot(l, y)
    plt.show()

def createHomoStats(startIdx = 0, maxFeatures = 1900):
    plotHomoStats(ORBprefix, startIdx, maxFeatures)
    plt.show()

def createHists(prefix):
    plt.subplot(1,2,1)
    plotNorms(prefix)
    plt.subplot(1,2,2)
    plotHellinger(prefix)
    plt.show()

plt.rcParams['axes.labelsize'] = 'large'
plt.rcParams['font.family'] = 'serif'
ORBprefix = "data/ORB-FS5-NF4119"
SIFTprefix = "data/SIFT-FS5-NF4119"
SURFprefix = "data/SURF-FS5-NF4119"
