import matplotlib
import matplotlib.pyplot as plt
import numpy as np

from skimage import img_as_float, exposure, morphology, segmentation
from skimage.filters import *
from skimage.feature import canny
from scipy.ndimage import imread, binary_fill_holes

def canny_segment(img, n_fill=0):
    img_f = img_as_float(img)
    edges = canny(img_f)
    fill = binary_fill_holes(edges)

    for k in range(0,n_fill):
        fill = binary_fill_holes(fill)

    return (fill, edges)

def watershed(img):
    elevation_map = sobel(img)
    markers = np.zeros_like(img)
    markers[img < 150] = 1
    markers[img > 200] = 2
    result = morphology.watershed(elevation_map, markers)

    return (result, elevation_map, markers)

def random(img):
    markers = np.zeros_like(img)
    markers[img < 100] = 1
    markers[img > 200] = 2
    labels = segmentation.random_walker(img, markers, beta=10, mode='bf')

    return(labels, markers)


def plot_img(img, ax_img, title=''):
    # Display image
    ax_img.imshow(img, cmap=plt.cm.gray)
    ax_img.set_axis_off()


def plot_hist(img, ax_hist, bins=256):
    #img = img_as_float(img)
    ax_hist.hist(img.ravel(), bins=bins, histtype='step', color='black')
    ax_hist.ticklabel_format(axis='y', style='scientific', scilimits=(0, 0))
    ax_hist.set_xlabel('Pixel intensity')
    ax_hist.set_xlim(0, 255)
    ax_hist.set_yticks([])

fig = plt.figure()

img = imread('infrared.jpg')

ax_img = plt.subplot(2,2,1)
ax_hist = plt.subplot(2,2,2)
ax_otsu = plt.subplot(2,2,3)
ax_close = plt.subplot(2,2,4)

(img_canny, edges) = canny_segment(img, 20)
(seg_watershed, elevation_map, markers_watershed) = watershed(img)
(labels_random, markers_random) = random(img)
otsu_global = threshold_otsu(img)
otsu_local = rank.otsu(img, morphology.disk(25))
img_adaptive = threshold_adaptive(img, 51)

closing_disk = morphology.disk(3)

plot_img(img, ax_img)
ax_img.set_title('Raw image')
plot_hist(img, ax_hist)
ax_hist.set_title('Image histogram')
plot_img(img > otsu_global, ax_otsu)
ax_otsu.set_title('Image > Otsu = {o}'.format(o=otsu_global))
plot_img(morphology.closing(img > otsu_global, closing_disk), ax_close)
ax_close.set_title('Morphological closing')

plt.show()
