# General Information
This package provides a MAVLINK interface to the Natural Point OptiTrack motion capture system at the "Slangerobotlab" in room B333. It also provides an interface to the AR Drone via MAVLINK. The package can be used to control the AR Drone with DUNE/Neptus. It has currently been tested to follow "goto" commands successfully. The package is implemented as an ROS package and has dependencies to a few other ROS packages. To build and use the package you need to have ROS installed. For installation instructions see http://wiki.ros.org/.

# Build instructions
The package has been developed using the *indigo* ROS distribution in the catkin (http://wiki.ros.org/catkin) build environment, but in principle should build and run on any ROS distribution newer than this. The only build dependency not included in the standard ROS libraries is the *libmavconn* package. If you are running Ubuntu this package is most likely available in your added ROS repositories. To install it, run
```sudo apt-get install ros-indigo-libmavconn```
Additionally, all nodes in this package require an OptiTrack motion capture driver to be running, this can be found in the *mocap-optitrack* ROS package. As of writing this README the package was not available in the indigo repositories. It can be built from source by adding the package to your catkin workspace. To add the OptiTrack driver package, run the following commands
```cd catkin_ws/src
git clone https://github.com/ros-drivers/mocap_optitrack.git```

## Build the package
To build the packages, check out this repository to your catkin workspace. For help on how to set up a catkin workspace see http://wiki.ros.org/catkin/Tutorials/create_a_workspace. With your catkin workspace setup, clone the repository into your workspace
```cd catkin_ws/src
git clone git@uavlab.itk.ntnu.no:uavlab/ros_optitrack_mavlink.git
cd ../ && catkin_make
```

# Running
## Motion capture and MAVLINK
There is a dedicated ROS launch file to start the motion capture driver together with the MAVLINK publisher node. To launch and set the tcp address and listening port run the following command with the 
```cd catkin_ws
source devel/setup.bash
roslaunch optitrack_mavlink mocap_mavlink.launch url:="tcp-l://129.241.208.70:5760"```

## AR Drone autopilot
### Get run dependencies
The autopilot has a few run dependencies. Depending on your environment, they might be available through your package manager. The ROS packages required are
- ardrone_autonomy (AR drone driver)
- tum_ardrone (AR drone autopilot)
- joy (joystick control of the AR drone)
- mocap_optitrack (motion capture driver, installation instructions already explained)
All dependencies can be built from source, but many of them can be found in the official Ubuntu ROS repositories. To install these on Ubuntu run
```sudo apt-get install ros-indigo-ardrone-autonomy ros-indigo-joy
```
The tum_ardrone package was not available in the repositories as of writing this README. So this can be built from source by adding it to your catkin workspace by running the following commands
```cd catkin_ws/src
git clone https://github.com/tum-vision/tum_ardrone.git
```
Unfortunately the *tum_ardrone* package includes a lot of unneccessary functionality in the form of computer vision algorithm implementations. This is not used in practice but it is still neccessary to build the entire package and link to its dependencies. To get the dependencies on Ubuntu run
```rosdep install tum_ardrone
sudo apt-get install freeglut3-dev
```
Where the *freeglut3-dev* package is needed to link to glut. If you are running Linux mint rosdep might fail to find the dependencies. You can fake your distribution by changing this to
```rosdep -os=ubuntu:trusty install tum_ardrone```
### Run the autopilot nodes
There is a launch file set up to run the autopilot. Simply run
```roslaunch optitrack_mavlink ardrone.launch```
