#!/usr/bin/env python

# ROS node to get 6-dof state from the mocap_optitrack driver and send it to the tum_ardrone ardrone autopilot
# Calculates velocities via finite difference using the ``lookupTwistFull()'' ROS function
# alf, 2015

import rospy
import tf
from geometry_msgs.msg import PoseStamped
from tum_ardrone.msg import filter_state
from tf.transformations import euler_from_quaternion
from math import pi

def getFilterState(pose, twist):
    pos = pose[0]
    orient_quat = pose[1]
    vel = twist[0]
    orient_euler = map(lambda r : 180*r/pi, euler_from_quaternion(orient_quat, axes='szyx'))
    state = filter_state(x = pos[0], y = pos[1], z = pos[2], yaw = orient_euler[0], pitch = orient_euler[1], roll = orient_euler[2], dx = vel[0], dy = vel[1], dz = vel[2], ptamState = filter_state.PTAM_BEST, scaleAccuracy=1)
    return state

def publishState(pose, twist, pub):
    state = getFilterState(pose, twist)
    pub.publish(state)

def listener():
    rospy.init_node('state_listener')
    statePublisher = rospy.Publisher('/ardrone/predictedPose', filter_state, queue_size=10)

    tfListener = tf.TransformListener()
    # Frame names are configured in the mocap_optitrack driver
    world_f = '/world'
    ardrone_f = '/Robot_1/base_link'
 
    rate = rospy.Rate(60)
    while not rospy.is_shutdown():
        try:
            # get pose
            pose = tfListener.lookupTransform(world_f, ardrone_f, rospy.Time(0))
            # black magic to get body velocities in world frame via finite differencing
            twist = tfListener.lookupTwistFull(ardrone_f, world_f, ardrone_f, (0,0,0), ardrone_f, rospy.Time(0), rospy.Duration(1.0/30))
            publishState(pose, twist, statePublisher)
        except (tf.LookupException, tf.ConnectivityException, tf.ExtrapolationException):
            continue
        rate.sleep()

if __name__ == '__main__':
    listener()
